function ys=dae12bv(tspan,y0,nint,params,I_vec)
% THIS FUNCTION IS A MODIFIED VERSION of function 
% ys=dae2(f,tspan,y0,nint,params) available at 
% http://www.mathworks.com/matlabcentral/fileexchange/28-differential-algebraic-equation-solvers
% 
% The function calls onto 
% jac_dae12_bv
% The function is called by
% solve_dae12bv
% 
%% Comments of initial function dae2:
%
% solves a set of differential algebraic equations (DAEs)
%        f(t,y,y')=0  where y'=dy/dt
% with a 2nd order method starting from y0 at time t0 and 
% finishing at time tfin where tspan=[t0 t1 ... tfin].
% y0 is a column vector, tspan is a row vector.
%
% The solution is returned at all the times in tspan.
% The time steps are diff(ts)/nint within the domain, error
% management is entirely up to the user via tspan and nint.
%
% The jacobians of f, namely k=df/dy and m=df/dy', must be 
% provided by f, at each time compute: [f,k,m]=func(t,y,y').
% Both m and k may be given as sparse matrices.
% If warnings of poor convergence occur, then the coded
% jacobians probably have errors.
%
% Params is a struct containing the parameters of the model - needed for
% the jacobian function.
%
% The initial state y0 should be consistent with the 
% algebraic part of the DAE, but if not consistent then 
% transient oscillations will appear as it works towards
% consistency.
%
% (c) Tony Roberts, 18 Aug 1998, aroberts@usq.edu.au

coder.extrinsic('warning')
warning('off','MATLAB:nearlySingularMatrix');

%% Check input format
assert(isrow(tspan) && isnumeric(tspan) && isreal(tspan) , 'tspan should be a row vector')
assert(iscolumn(y0) && isnumeric(y0) && isreal(y0), 'y0 should be a column vector')
assert(all(~isnan(y0)),'y0 contains NaN entries, this should never happen')
assert(isscalar(nint) && isreal(nint) && nint>0 , 'nint should be a positive scalar')
assert(isnumeric(I_vec) && isreal(I_vec),'I_vec should be real numbers')


I_temp1 = [NaN NaN]; I_temp2 = NaN; % Initialise these for mex - don't use finite values

% Check format if I_vec is single value or vector
if isscalar(I_vec)==true
    int_sw = false;
    I_temp1 = [I_vec I_vec];
    I_temp2 = I_vec;
elseif isvector(I_vec)==true
    int_sw = true;
    assert(all(size(I_vec)==size(tspan)),'I_vec should be scalar or a vector the size as tspan')
else
    error('I_vec should be scalar or a vector the size as tspan');
end

%%
nout=length(tspan);
ndim=length(y0);
ys=zeros(ndim,nout)*NaN;
newtol=1e-6;
newtmax=10;
newtit=zeros(1,newtmax);
% weights for BDF est of deriv
wd=[0 1 1/2]; 
wds=sum(wd);

e12 = zeros(12,12);
eps2 = 10*eps;

% to allow for varying spaced output, fit a spline
% and solve in s=[1,nts] rather than in t
% dt is dt/ds at each time s
nts=1+nint*(nout-1);
if nts<3, disp('ERROR: dae2 needs at least two steps'), return, end
ts=spline(1:nint:nts,tspan,(1:nts));
dt=spline(1:nint:nts,tspan,(1:nts)+1e-7);
dt=(dt-ts)/1e-7;

% Interp current
if int_sw==true, I_new = interp1(tspan,I_vec,ts); I_vec = I_new(:).'; end  %Odd syntax is due to mex and interp1

% initialise by solving first two steps together assuming
% a quadratic between them (zero third difference).
yy=zeros(ndim,2); % initial guess
y=[y0 yy]; 

for newt=1:newtmax
    % extrapolate
    y2=mycumsum(y);
    y3=mycumsum(y2);
    % evaluate residuals
    if int_sw==true, I_temp1 = [I_vec(2); I_vec(3)]; end
    [f2,k2,m2]=jac_dae12bv(ts(2),y2(:,1),y2*wd'/dt(2),params,I_temp1(1),e12);
    [f3,k3,m3]=jac_dae12bv(ts(3),y3(:,1),y3*wd'/dt(3),params,I_temp1(2),e12);
    % solve simultaneous equations
    yy(:)=-[  k2+m2/dt(2)   k2+1.5*m2/dt(2)
        2*k3+m3/dt(3) 3*k3+2.5*m3/dt(3) ]\[f2;f3];
    y(:,2:3)=y(:,2:3)+yy;
    if max(abs(yy(:)))<newtol*max(abs(y(:))), break, end
end
% output as requested
ys(:,1)=y(:,1);

y=mycumsum(y);
if nint==1
    ys(:,2)=y(:,1);
end

y=mycumsum(y);
if nint<=2
    ys(:,4-nint)=y(:,1);
end

% take second order steps over domain
ones_wd = ones(size(wd));
for n=4:nts
    h=dt(n);
    if int_sw==true, I_temp2 = I_vec(n); end
    y=mycumsum(y); % extrapolate a guess
    for newt=1:newtmax
        %% Check validity of y
        if all(y(1,1)<0)
            warning('Cell has been over discharged')
            return
        elseif all(y(3,1)<0)
            warning('cell has been over charged')
            return
        end
        
        if all(y(2,1)<eps2)
            if  all(y(1,1)<y(3,1)) && all((y(6,1)+y(7,1))>-eps2)
                warning('Cell has been over discharged')
                return
            elseif all(y(1,1)>y(3,1))  && all((y(6,1)+y(7,1))<eps2)
                warning('Cell has been over charged')
                return
            end
        end
        %% End validity check
        
        [f1,k1,m1]=jac_dae12bv(ts(n),y(:,1),y*wd'/h,params,I_temp2,e12);
        w=-(wds*m1/h+k1)\f1;
        y=y+w*ones_wd;
        
        if max(abs(w))<newtol*max(abs(y(:))); break; end
    end
    
    newtit(newt)=newtit(newt)+1;
    if rem(n-1,nint)==0
        ys(:,1+(n-1)/nint)=y(:,1);
    end
end

% check on how many Newtonian iterations were required
newtm=sum((1:newtmax).*newtit)/sum(newtit);
if newtm>newtmax/2,
    warning('poor or no convergence in dae12')
end
