function [f,k,m]=jac_dae12bv(~,w,wd,params,I,e12)
% Populates the jacobian and mass matrix for a system of 12
% equations and 12 variables
% 
% The function call onto - nobody
% The function is called by
% dae12bv
%
% Subject to Creative Commons Attribution 4.0 International Public License
% see license.txt
% (c) Monica Marinescu, 23 Sept 2015, monica.marinescu@imperial.ac.uk
%
%% In terms of variable names, the system and jacobian look like this:
%
% f=[-S8d - IH/4/params.F*8*params.mm - params.ks*S8
%    -S4d + IH/4/params.F*8*params.mm + params.ks*S8 - IL/4/params.F*4*params.mm
%    -S2d + IL/4/params.F*2*params.mm
%    -Sd  + IL/4/params.F*2*params.mm - Sp/params.rhoS/params.vol*params.kp*(S-params.Ksp)
%    -Spd + Sp/params.rhoS/params.vol*params.kp*(S-params.Ksp)
%    -I + IH + IL
%    IH + 2*params.iHr * sinh(etaH*4*params.F/2/params.R/params.T)
%    IL + 2*params.iLr * sinh(etaL*4*params.F/2/params.R/params.T)
%    -etaH + V-ENH
%    -etaL + V-ENL
%    -ENH + params.EH0 + params.R*params.T/4/params.F*log(0.7296*S8/S4^2)
%    -ENL + params.EL0 + params.R*params.T/4/params.F*log(0.0665*S4/S^2/S2)];
% 
% k=[-params.ks 0 0 0 0 -8*params.mm/4/params.F 0 0 0 0 0 0
%     params.ks 0 0 0 0 8*params.mm/4/params.F -4*params.mm/4/params.F 0 0 0 0 0
%     0 0 0 0 0 0 2*params.mm/4/params.F 0 0 0 0 0
%     0 0 0 -Sp*params.kp/params.rhoS/params.vol -params.kp/params.rhoS/params.vol*(S-params.Ksp) 0 2*params.mm/4/params.F 0 0 0 0 0
%     0 0 0 Sp/params.rhoS/params.vol*params.kp params.kp/params.rhoS/params.vol*(S-params.Ksp) 0 0 0 0 0 0 0
%     0 0 0 0 0 1 1 0 0 0 0 0
%     0 0 0 0 0 1 0 0 2*params.iHr*cosh(etaH*4*params.F/2/params.R/params.T)*4*params.F/2/params.R/params.T 0 0 0
%     0 0 0 0 0 0 1 0 0 2*params.iLr*cosh(etaL*4*params.F/2/params.R/params.T)*4*params.F/2/params.R/params.T 0 0 
%     0 0 0 0 0 0 0 1 -1 0 -1 0
%     0 0 0 0 0 0 0 1 0 -1 0 -1
%     params.R*params.T/4/params.F/S8 -2*params.R*params.T/4/params.F/S4 0 0 0 0 0 0 0 0 -1 0
%     0 params.R*params.T/4/params.F/S4 -params.R*params.T/4/params.F/S2 -2*params.R*params.T/4/params.F/S 0 0 0 0 0 0 0 -1];
%
% The order of variables:
% S8=w(1); S4=w(2); S2=w(3); S=w(4); Sp=w(5); IH=w(6); IL=w(7); V=w(8); etaH=w(9); etaL=w(10); ENH=w(11); ENL=w(12);
%
% and derivatives:
% S8d=wd(1); S4d=wd(2); S2d=wd(3); Sd=wd(4); Spd=wd(5);

%% Assertions
assert(isreal(w) && isreal(wd),'Jacobian generator: A complex set of variables have been passed.')
assert( w(1)> 0 && w(2)>0 && w(3)>0 && w(4)>0, 'Jacobian generator: Concentrations of sulphur are negative' )

%% Define useful local variables

a = (params.R*params.T)./(4*params.F);
b = 1./(2*a);
c = params.kp./(params.rhoS*params.vol);
d = (2*params.mm)./(4*params.F);
d2 = 2*d;
d4 = 4*d;

cm = -c;
m2a = -2*a;

p1 = 1;
n1 = -1;

%% Express system of equations as 0 = F(t,x,x'), with x = w, x' = wd

f=[-wd(1) - d4*w(6) - params.ks*w(1)
   -wd(2) + d4*w(6) + params.ks*w(1) - d2*w(7)
   -wd(3) + d*w(7)
   -wd(4) + d*w(7) - c*w(5)*(w(4)-params.Ksp)
   -wd(5) + c*w(5)*(w(4) - params.Ksp)
   -I + w(6) + w(7)
   w(6) + 2*params.iHr * sinh(w(9) * b)
   w(7) + 2*params.iLr * sinh(w(10)* b)
   -w(9)  + w(8) - w(11)
   -w(10) + w(8) - w(12)
   -w(11) + params.EH0 + a * log(0.7296*w(1)/w(2)^2)
   -w(12) + params.EL0 + a * log(0.0665*w(2)/w(4)^2/w(3))]; 


%% Calculate mass matrix


m = e12; % default values: 12x12 matrix of zeroes

m(1,1) = n1;
m(2,2) = n1;
m(3,3) = n1;
m(4,4) = n1;
m(5,5) = n1;

%% Calculate Jacobian matrix

k=e12; % default values: 12x12 matrix of zeroes

k(1,1) = -params.ks;
k(1,6) = -d4;
k(2,1) = params.ks;
k(2,6) = d4;
k(2,7) = -d2;
k(3,7) = d;
k(4,4) = cm*w(5);
k(4,5) = cm*(w(4)-params.Ksp);
k(4,7) = d;
k(5,4) = c*w(5);
k(5,5) = c*(w(4)-params.Ksp);
k(6,6) = p1;
k(6,7) = p1;
k(7,6) = p1;
k(7,9) = 2*params.iHr*cosh(w(9) * b)*b;
k(8,7) = p1;
k(8,10) = 2*params.iLr*cosh(w(10) * b)*b;
k(9,8) = p1;
k(9,9) = n1;
k(9,11) = n1;
k(10,8) = p1;
k(10,10) = n1;
k(10,12) = n1;
k(11,1) = a./w(1);
k(11,2) = m2a./w(2);
k(11,11) = n1;
k(12,2) = a./w(2);
k(12,3) = -a./w(3);
k(12,4) = m2a./w(4);
k(12,12) = n1;


