% Script for constant current discharge
%
% It solves 12 equations for 12 variables:
% $S_8$, $S_4$, $S_2$, S, $S_p$, $i_H$, $i_L$, V, $\eta_H$, $\eta_L$,
% $ENH$, $ENL$
%
% The calculated values are stored as rows in the matrix 'w'
% for example, w(5,1000) is the value for the 5th variable, $S_p$, at the 
% 5000th time step 
%
% The script calls onto the following functions:
% make_params, make_ic_disch_12, solve_dae12bv, plot_data_1run
%
% Example plot data script and save data included at the end
% 
% Subject to Creative Commons Attribution 4.0 International Public License
% see license.txt
% (c) Monica Marinescu, 23 Sept 2015, monica.marinescu@imperial.ac.uk


close all; clear all
addpath('lib/')

%% Initialise problem
% ppt and BV are boolean parameters: true for model including the
% precipitation and Butler-Volmer, false for model without either of
% these

ppt = true;
BV = true;

% Load parameters
params = make_params('ppt',ppt,'BV',BV);

% Constant current value (negative = charge) and system sulfur mass
% for these values and this model the current corresponds to
% 0.85 (0.25C), 1.7 (0.5C), 3.4 (1C), 6.8 (2C)
I = 1.7; % A
ms = 2.7; % g S

% Time
% if time_end is longer than the time it takes to charge, then the
% solver gives a warning and terminates the program
time_end = 6800;% s 6701
time_N = ceil(20000); % maximum number of steps
ts=linspace(0, time_end, time_N);


% current axis for constant current
I_vec = I;
% alternatively, I_vec can be a vector specifying the value of current at
% every point in ts

%% Construct initial conditions

% initial conditions - discharge from fully charged
Vin = 2.4; % V
S8in = 0.99*ms;
Spin = 0.000001*ms;

% calculate the initial conditions for all other variables
w0 = make_ic_disch_12(params,I_vec(1), Vin, S8in, Spin);

%% Solve problem - with or without mex
% using the mex code (default) is much faster

% w=solve_dae12bv(ts,w0,4,params,I_vec,'mex',false);
w=solve_dae12bv(ts,w0,4,params,I_vec);


%% Plot data - example
plot_data_1run;

